/**
  @file main.c
  @brief
  FT90x Ethernet to I2C EEPROM Memory Bridge Example.

 */
/*
 * ============================================================================
 * History
 * =======
 * 27 May 2016 : Created
 *
 * (C) Copyright 2014-2016, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */


/* INCLUDES ************************************************************************/

#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <ft900.h>
#include <ft900_uart_simple.h>
#include <ft900_eth.h>
#include <ft900_startup_dfu.h>

/* UART support for printf output. */
#include "tinyprintf.h"

#include "net.h"

#include <lwip\apps\httpd.h>

#define BRIDGE_DEBUG
#ifdef BRIDGE_DEBUG
#define BRIDGE_DEBUG_PRINTF(...) do {tfp_printf(__VA_ARGS__);} while (0)
#else
#define BRIDGE_DEBUG_PRINTF(...)
#endif

#define BLOCK_SIZE 32
/* CONSTANTS ***********************************************************************/


/* GLOBAL VARIABLES ****************************************************************/
char hostname[64] = {'I','2','C','B','r','i','d','g','e',0};
char i2c_ok[3] = {'O','K',0};
char i2c_fail[5] = {'F','A','I','L',0};

uint8_t i2c_status;
uint8_t i2cadd_var = 0xA0;
uint8_t eeprom_size = BLOCK_SIZE;
uint8_t eeprom_buffer[BLOCK_SIZE] = {0};
uint8_t read_add = 0;
uint8_t write_add = 0;

/* LOCAL VARIABLES *****************************************************************/

/**
 @brief Millisecond counter
 @details Count-up timer to provide the elapsed time for network operations.
 */
static uint32_t milliseconds = 0;

/**
 @brief lwIP timer counter
 @details Count-down timer to provide the a periodic 'tick' for lwIP timer
 	 operations.
 */
static uint16_t lwip_timer = 0;

static uint16_t stats_timer = 0;


/**
 @brief Structure to hold network settings.
 @details This is used to both keep current settings in use and
 	 to store persistently in Flash.
 */
uint8_t altmacaddr[6] = {0};

/**
 @brief Watchdog keep alive flag.
 @details Set to zero to initiate a watchdog reset.
 */
int do_wdg_kick = 1;

/* MACROS **************************************************************************/

/* LOCAL FUNCTIONS / INLINES *******************************************************/

int8_t ee_read(const uint8_t location, uint8_t *data, const uint8_t len);
int8_t ee_write(const uint8_t location, uint8_t *data, const uint8_t len);

/** @name tfp_putc
 *  @details Machine dependent putc function for tfp_printf (tinyprintf) library.
 *  @param p Parameters (machine dependent)
 *  @param c The character to write
 */
void tfp_putc(void* p, char c)
{
	uart_write((ft900_uart_regs_t*)p, (uint8_t)c);
}

/** @brief Returns the current millisecond country
 *  @returns A count of milliseconds
 */
uint32_t millis(void)
{
	return milliseconds;
}

void ISR_timer(void)
{
	if (timer_is_interrupted(timer_select_a))
	{
		milliseconds++;

		if (lwip_timer)
		{
			lwip_timer--;
		}

		if (stats_timer)
		{
			stats_timer--;
		}
	}
}

// HTTPD Server Side Include Handler
#define FOREACH_SSI(MAKE_SSI) \
		MAKE_SSI(mac)   \
		MAKE_SSI(altmac)   \
		MAKE_SSI(altmaca)   \
		MAKE_SSI(hn)   \
		MAKE_SSI(ip)  \
		MAKE_SSI(gw)   \
		MAKE_SSI(nm)  \
		MAKE_SSI(dhcp)  \
		MAKE_SSI(i2cadd)  \
		MAKE_SSI(eesize)  \
		MAKE_SSI(startr)  \
		MAKE_SSI(startw)  \
		MAKE_SSI(i2cstat)  \
		MAKE_SSI(byte0)  \
		MAKE_SSI(byte1)  \
		MAKE_SSI(byte2)  \
		MAKE_SSI(byte3)  \
		MAKE_SSI(byte4)  \
		MAKE_SSI(byte5)  \
		MAKE_SSI(byte6)  \
		MAKE_SSI(byte7)  \
		MAKE_SSI(byte8)  \
		MAKE_SSI(byte9)  \
		MAKE_SSI(bytea)  \
		MAKE_SSI(byteb)  \
		MAKE_SSI(bytec)  \
		MAKE_SSI(byted)  \
		MAKE_SSI(bytee)  \
		MAKE_SSI(bytef)  \
		MAKE_SSI(byte10)  \
		MAKE_SSI(byte11)  \
		MAKE_SSI(byte12)  \
		MAKE_SSI(byte13)  \
		MAKE_SSI(byte14)  \
		MAKE_SSI(byte15)  \
		MAKE_SSI(byte16)  \
		MAKE_SSI(byte17)  \
		MAKE_SSI(byte18)  \
		MAKE_SSI(byte19)  \
		MAKE_SSI(byte1a)  \
		MAKE_SSI(byte1b)  \
		MAKE_SSI(byte1c)  \
		MAKE_SSI(byte1d)  \
		MAKE_SSI(byte1e)  \
		MAKE_SSI(byte1f)  \
		MAKE_SSI(total)

#define GENERATE_SSI_ENUM(ENUM) ssiTag_##ENUM,
#define GENERATE_SSI_STRING(STRING) #STRING,

enum SSI_ENUM {
	FOREACH_SSI(GENERATE_SSI_ENUM)
};

static const char *ssiTags[] = {
		FOREACH_SSI(GENERATE_SSI_STRING)
};

u16_t app_httpd_ssi_cb(int iIndex, char *pcInsert, int iInsertLen)
{
	int len = 0;
	ip_addr_t ia;
	uint8_t *pval;
	char rstring[32];
	char *enstring = "";
	char *altmace = "";
	char *dstring;
	char *dstring1 = "disabled";
	char *dstring2;
	const char *nstring;
	struct eeprom_net_config getval;

	net_get_eeprom(&getval);

	if (getval.dhcp == 0)
	{
		enstring = "disabled";
	}

	if (getval.alt_mac == 0)
	{
		altmace = "disabled";
	}

	if (iIndex < ssiTag_total)
	{
		nstring = ssiTags[iIndex];

		switch (iIndex)
		{
		case ssiTag_mac:
			pval = net_get_mac(rstring);
			sprintf(rstring, "%02X:%02X:%02X:%02X:%02X:%02X",
					pval[0], pval[1], pval[2], pval[3], pval[4], pval[5]);
			len = sprintf(pcInsert, " <input type=\"text\" name=\"%s\" maxlength=\"17\" value=\"%s\" disabled>", nstring, rstring);
			break;
		case ssiTag_altmaca:
			pval = altmacaddr;
			sprintf(rstring, "%02X:%02X:%02X:%02X:%02X:%02X",
					pval[0], pval[1], pval[2], pval[3], pval[4], pval[5]);
			len = sprintf(pcInsert, " <input type=\"text\" name=\"%s\" value=\"%s\" maxlength=\"17\" %s>", nstring, rstring, altmace);
			break;
		case ssiTag_altmac:
			if (getval.alt_mac)
			{
				dstring2 = "checked";
			}
			len = sprintf(pcInsert, " <input type=\"checkbox\" name=\"%s\" %s>", nstring, dstring2);
			break;
		case ssiTag_hn:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%s\" maxlength=\"64\" %s>", nstring, hostname, dstring1);
			break;
		case ssiTag_ip:
			ia = getval.ip;
			dstring = ipaddr_ntoa(&ia);
			len = sprintf(pcInsert, " <input type=\"text\" name=\"%s\" value=\"%s\" maxlength=\"16\" %s>", nstring, dstring, enstring);
			break;
		case ssiTag_gw:
			ia = getval.gw;
			dstring = ipaddr_ntoa(&ia);
			len = sprintf(pcInsert, " <input type=\"text\" name=\"%s\" value=\"%s\" maxlength=\"16\" %s>", nstring, dstring, enstring);
			break;
		case ssiTag_nm:
			ia = getval.mask;
			dstring = ipaddr_ntoa(&ia);
			len = sprintf(pcInsert, " <input type=\"text\" name=\"%s\" value=\"%s\" maxlength=\"16\" %s>", nstring, dstring, enstring);
			break;
		case ssiTag_dhcp:
			if (getval.dhcp)
			{
				dstring = "checked";
			}
			len = sprintf(pcInsert, " <input type=\"checkbox\" name=\"%s\" %s>", nstring, dstring);
			break;
		case ssiTag_i2cadd:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%X\" size=\"3\" maxlength=\"2\">", nstring, i2cadd_var);
			break;
		case ssiTag_eesize:
			len = sprintf(pcInsert, "<input type=\"number\" name=\"%s\" value=\"%d\" size=\"3\" %s>", nstring, eeprom_size, dstring1);
			break;
		case ssiTag_startr:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%x\" size=\"3\" maxlength=\"2\">", nstring, read_add);
			break;
		case ssiTag_startw:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%x\" size=\"3\" maxlength=\"2\">", nstring, write_add);
			break;
		case ssiTag_i2cstat:
			if (i2c_status == 0)
			{
				len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%s\" size=\"3\" %s>", nstring, i2c_ok, dstring1);
			}
			else
			{
				len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%s\" size=\"3\" %s>", nstring, i2c_fail, dstring1);
			}
			break;
		case ssiTag_byte0:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[0]);
			break;
		case ssiTag_byte1:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[1]);
			break;
		case ssiTag_byte2:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[2]);
			break;
		case ssiTag_byte3:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[3]);
			break;
		case ssiTag_byte4:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[4]);
			break;
		case ssiTag_byte5:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[5]);
			break;
		case ssiTag_byte6:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[6]);
			break;
		case ssiTag_byte7:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[7]);
			break;
		case ssiTag_byte8:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[8]);
			break;
		case ssiTag_byte9:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[9]);
			break;
		case ssiTag_bytea:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[10]);
			break;
		case ssiTag_byteb:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[11]);
			break;
		case ssiTag_bytec:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[12]);
			break;
		case ssiTag_byted:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[13]);
			break;
		case ssiTag_bytee:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[14]);
			break;
		case ssiTag_bytef:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[15]);
			break;
		case ssiTag_byte10:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[16]);
			break;
		case ssiTag_byte11:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[17]);
			break;
		case ssiTag_byte12:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[18]);
			break;
		case ssiTag_byte13:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[19]);
			break;
		case ssiTag_byte14:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[20]);
			break;
		case ssiTag_byte15:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[21]);
			break;
		case ssiTag_byte16:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[22]);
			break;
		case ssiTag_byte17:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[23]);
			break;
		case ssiTag_byte18:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[24]);
			break;
		case ssiTag_byte19:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[25]);
			break;
		case ssiTag_byte1a:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[26]);
			break;
		case ssiTag_byte1b:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[27]);
			break;
		case ssiTag_byte1c:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[28]);
			break;
		case ssiTag_byte1d:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[29]);
			break;
		case ssiTag_byte1e:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[30]);
			break;
		case ssiTag_byte1f:
			len = sprintf(pcInsert, "<input type=\"text\" name=\"%s\" value=\"%02X\" size=\"1\" maxlength=\"2\">", nstring, eeprom_buffer[31]);
			break;
		}
	}

	return len;
}

// HTTPD CGI Handlers

const char *app_httpd_cgi_net(int iIndex, int iNumParams, char *pcParam[],
		char *pcValue[])
{
	int i;
	uint8_t mac_new[6];
	int val;
	char *strnptr;
	struct eeprom_net_config setval;

	// Checkboxes not always sent if unchecked.
	net_get_eeprom(&setval);
	setval.dhcp = 0;
	setval.alt_mac = 0;

	for(i = 0; i < iNumParams; i++)
	{
		if(!strcmp(pcParam[i], ssiTags[ssiTag_altmac]))
		{
			setval.alt_mac = 1;
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_altmaca]))
		{
			strnptr = pcValue[i];
			for (val = 0; val < 6; val++)
			{
				mac_new[val] = strtoul(strnptr, &strnptr, 16);
				// If the string from the web page is escaped then skip the escaped ':' (%3A).
				if (*strnptr == '%')
				{
					strnptr+=2;
				}
				altmacaddr[val] = mac_new[val];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_hn]))
		{
			strcpy(hostname, pcValue[i]);
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_dhcp]))
		{
			setval.dhcp = 1;
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_ip]))
		{
			ipaddr_aton(pcValue[i], &setval.ip);
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_gw]))
		{
			ipaddr_aton(pcValue[i], &setval.gw);
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_nm]))
		{
			ipaddr_aton(pcValue[i], &setval.mask);
		}
	}

	BRIDGE_DEBUG_PRINTF("Net Config:\n");

	if(setval.alt_mac == 1)
			{
		BRIDGE_DEBUG_PRINTF("  MAC = %02X:%02X:%02X:%02X:%02X:%02X\n",
				altmacaddr[0], altmacaddr[1],
				altmacaddr[2], altmacaddr[3],
				altmacaddr[4], altmacaddr[5]);
			}

		BRIDGE_DEBUG_PRINTF("  Hostname = \"%s\"\n", hostname);
		BRIDGE_DEBUG_PRINTF("  DHCP = %c\n", setval.dhcp?'Y':'N');
		BRIDGE_DEBUG_PRINTF("  ipaddr = %s\n", ipaddr_ntoa(&setval.ip));
		BRIDGE_DEBUG_PRINTF("  gwaddr = %s\n", ipaddr_ntoa(&setval.gw));
		BRIDGE_DEBUG_PRINTF("  netmask = %s\n", ipaddr_ntoa(&setval.mask));

		net_update_eeprom(&setval);

		//Update alternative MAC address
		if(setval.alt_mac == 1)
		{
		i2c_status = ee_write(NET_EEPROM_OFFSET_ALT_MACADDRESS, &altmacaddr, 6);
		}

	// Force a reboot through the watchdog timer.
	do_wdg_kick = 0;

	return "/restart.shtm";
}

const char *app_httpd_cgi_i2cread(int iIndex, int iNumParams, char *pcParam[],
		char *pcValue[])
{
	int i, j, add;

	for(i = 0; i < iNumParams; i++)
	{
		if(!strcmp(pcParam[i], ssiTags[ssiTag_i2cadd]))
		{
			if (sscanf(pcValue[i], "%X", &add))
			{
				i2cadd_var = (uint8_t) add;
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_eesize]))
		{
			eeprom_size = atoi(pcValue[i]);
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_startr]))
		{
			if (sscanf(pcValue[i], "%X", &add))
			{
				read_add = (uint8_t) add;
			}
		}
	}

	for (j=0; j<eeprom_size; j++)
	{
		eeprom_buffer[j] = 0;
	}

	i2c_status = ee_read(read_add, &eeprom_buffer, eeprom_size);

	BRIDGE_DEBUG_PRINTF("I2C EEPROM Read status = %d\r\n", i2c_status);

	return "/index.shtm";
}

const char *app_httpd_cgi_i2cwrite(int iIndex, int iNumParams, char *pcParam[],
		char *pcValue[])
{
	int i, i2c_add, start_add, data[32];

	for(i = 0; i < iNumParams; i++)
	{
		if(!strcmp(pcParam[i], ssiTags[ssiTag_i2cadd]))
		{
			if (sscanf(pcValue[i], "%X", &i2c_add))
			{
				i2cadd_var = (uint8_t) i2c_add;
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_eesize]))
		{
			eeprom_size = atoi(pcValue[i]);
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_startw]))
		{
			if (sscanf(pcValue[i], "%X", &start_add))
			{
				write_add = (uint8_t) start_add;
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte0]))
		{
			if (sscanf(pcValue[i], "%X", &data[0]))
			{
				eeprom_buffer[0] = (uint8_t) data[0];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte1]))
		{
			if (sscanf(pcValue[i], "%X", &data[1]))
			{
				eeprom_buffer[1] = (uint8_t) data[1];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte2]))
		{
			if (sscanf(pcValue[i], "%X", &data[2]))
			{
				eeprom_buffer[2] = (uint8_t) data[2];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte3]))
		{
			if (sscanf(pcValue[i], "%X", &data[3]))
			{
				eeprom_buffer[3] = (uint8_t) data[3];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte4]))
		{
			if (sscanf(pcValue[i], "%X", &data[4]))
			{
				eeprom_buffer[4] = (uint8_t) data[4];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte5]))
		{
			if (sscanf(pcValue[i], "%X", &data[5]))
			{
				eeprom_buffer[5] = (uint8_t) data[5];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte6]))
		{
			if (sscanf(pcValue[i], "%X", &data[6]))
			{
				eeprom_buffer[6] = (uint8_t) data[6];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte7]))
		{
			if (sscanf(pcValue[i], "%X", &data[7]))
			{
				eeprom_buffer[7] = (uint8_t) data[7];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte8]))
		{
			if (sscanf(pcValue[i], "%X", &data[8]))
			{
				eeprom_buffer[8] = (uint8_t) data[8];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte9]))
		{
			if (sscanf(pcValue[i], "%X", &data[9]))
			{
				eeprom_buffer[9] = (uint8_t) data[9];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_bytea]))
		{
			if (sscanf(pcValue[i], "%X", &data[10]))
			{
				eeprom_buffer[10] = (uint8_t) data[10];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byteb]))
		{
			if (sscanf(pcValue[i], "%X", &data[11]))
			{
				eeprom_buffer[11] = (uint8_t) data[11];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_bytec]))
		{
			if (sscanf(pcValue[i], "%X", &data[12]))
			{
				eeprom_buffer[12] = (uint8_t) data[12];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byted]))
		{
			if (sscanf(pcValue[i], "%X", &data[13]))
			{
				eeprom_buffer[13] = (uint8_t) data[13];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_bytee]))
		{
			if (sscanf(pcValue[i], "%X", &data[14]))
			{
				eeprom_buffer[14] = (uint8_t) data[14];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_bytef]))
		{
			if (sscanf(pcValue[i], "%X", &data[15]))
			{
				eeprom_buffer[15] = (uint8_t) data[15];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte10]))
		{
			if (sscanf(pcValue[i], "%X", &data[16]))
			{
				eeprom_buffer[16] = (uint8_t) data[16];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte11]))
		{
			if (sscanf(pcValue[i], "%X", &data[17]))
			{
				eeprom_buffer[17] = (uint8_t) data[17];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte12]))
		{
			if (sscanf(pcValue[i], "%X", &data[18]))
			{
				eeprom_buffer[18] = (uint8_t) data[18];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte13]))
		{
			if (sscanf(pcValue[i], "%X", &data[19]))
			{
				eeprom_buffer[19] = (uint8_t) data[19];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte14]))
		{
			if (sscanf(pcValue[i], "%X", &data[20]))
			{
				eeprom_buffer[20] = (uint8_t) data[20];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte15]))
		{
			if (sscanf(pcValue[i], "%X", &data[21]))
			{
				eeprom_buffer[21] = (uint8_t) data[21];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte16]))
		{
			if (sscanf(pcValue[i], "%X", &data[22]))
			{
				eeprom_buffer[22] = (uint8_t) data[22];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte17]))
		{
			if (sscanf(pcValue[i], "%X", &data[23]))
			{
				eeprom_buffer[23] = (uint8_t) data[23];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte18]))
		{
			if (sscanf(pcValue[i], "%X", &data[24]))
			{
				eeprom_buffer[24] = (uint8_t) data[24];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte19]))
		{
			if (sscanf(pcValue[i], "%X", &data[25]))
			{
				eeprom_buffer[25] = (uint8_t) data[25];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte1a]))
		{
			if (sscanf(pcValue[i], "%X", &data[26]))
			{
				eeprom_buffer[26] = (uint8_t) data[26];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte1b]))
		{
			if (sscanf(pcValue[i], "%X", &data[27]))
			{
				eeprom_buffer[27] = (uint8_t) data[27];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte1c]))
		{
			if (sscanf(pcValue[i], "%X", &data[28]))
			{
				eeprom_buffer[28] = (uint8_t) data[28];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte1d]))
		{
			if (sscanf(pcValue[i], "%X", &data[29]))
			{
				eeprom_buffer[29] = (uint8_t) data[29];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte1e]))
		{
			if (sscanf(pcValue[i], "%X", &data[30]))
			{
				eeprom_buffer[30] = (uint8_t) data[30];
			}
		}
		else if(!strcmp(pcParam[i], ssiTags[ssiTag_byte1f]))
		{
			if (sscanf(pcValue[i], "%X", &data[31]))
			{
				eeprom_buffer[31] = (uint8_t) data[31];
			}
		}
	}

	i2c_status = ee_write(write_add, &eeprom_buffer, eeprom_size);

	BRIDGE_DEBUG_PRINTF("I2C EEPROM Write status = %d\r\n", i2c_status);

	return "/index.shtm";
}

/** EEPROM Read routine
 *  @param location The location to start reading
 *  @param data A pointer to the data to read into
 *  @param len The size of data to read */
int8_t ee_read(const uint8_t location, uint8_t *data, const uint8_t len)
{
	/* Set the I2C Master pins to channel 1 */
	sys_i2c_swop(1);
	i2cm_init(I2CM_NORMAL_SPEED, 10000);

	return i2cm_read(i2cadd_var, location, data, len);

	/* Set the I2C Master pins back to channel 0 */
	i2cm_init(I2CM_NORMAL_SPEED, 100000);
	sys_i2c_swop(0);
}

/** EEPROM Write routine
 *  @param location The location to start writing
 *  @param data A pointer to the data to write
 *  @param len The size of data to write */
int8_t ee_write(const uint8_t location, uint8_t *data, const uint8_t len)
{
	uint8_t i;
	int8_t retval = 0;

	/* Set the I2C Master pins to channel 1 */
	sys_i2c_swop(1);
	i2cm_init(I2CM_NORMAL_SPEED, 10000);

	/* Transfer a block of data by carrying out a series of byte writes
       and waiting for the write to complete */
	for (i = 0; (i < len) && (retval == 0); ++i)
	{
		retval = i2cm_write(i2cadd_var, location+i, data+i, 1);

		delayms(5); // Wait a write cycle time
	}

	/* Set the I2C Master pins back to channel 0 */
	i2cm_init(I2CM_NORMAL_SPEED, 100000);
	sys_i2c_swop(0);

	return retval;
}

const tCGI cgiHandler[] = {
		{"/net", app_httpd_cgi_net},
		{"/i2cread", app_httpd_cgi_i2cread},
		{"/i2cwrite", app_httpd_cgi_i2cwrite},
};



void i2c_bridge(void)
{
	ip_addr_t ip_addr = {0};

	wdt_kick();

	// Open the lwIP library and ask for a DHCP address.
	if (net_init(ip_addr, ip_addr, ip_addr, 0, hostname) == ERR_OK)
	{
		int started = 0;

		while (1)
		{
			if (net_is_link_up())
			{
				net_tick();

				if (!started)
				{
					started = net_is_up();
					if (started)
					{
						httpd_init();
						http_set_ssi_handler(app_httpd_ssi_cb, ssiTags, ssiTag_total);
						http_set_cgi_handlers(cgiHandler, sizeof(cgiHandler)/sizeof(tCGI));
					}
				}
			}

			if (do_wdg_kick)
			{
				wdt_kick();
			}

			if (stats_timer == 0)
			{
				//stats_display();
				//netif_stats_display_ft900();
				stats_timer = 10000;
			}

			if (lwip_timer == 0)
			{
				// Check for ethernet disconnection.
				if (net_is_link_up())
				{
					if (!ethernet_is_link_up())
					{
						net_set_link_down();
						BRIDGE_DEBUG_PRINTF("Ethernet disconnected.\r\n");
					}
				}
				else
				{
					if (ethernet_is_link_up())
					{
						net_set_link_up();
						BRIDGE_DEBUG_PRINTF("Ethernet connected.\r\n");
					}
				}

				// Check lwIP timeouts every 250 ms.
				sys_check_timeouts();
				lwip_timer = 250;
			}
		}
	}
	// Disconnected from host.
}


/* FUNCTIONS ***********************************************************************/

int main(void)
{
	sys_reset_all();

	STARTUP_DFU();

	/* Enable the UART Device... */
	sys_enable(sys_device_uart0);
	/* Make GPIO48 function as UART0_TXD and GPIO49 function as UART0_RXD... */
	gpio_function(48, pad_uart0_txd); /* UART0 TXD */
	gpio_function(49, pad_uart0_rxd); /* UART0 RXD */

	/* Enable tfp_printf() functionality... */
	init_printf(UART0, tfp_putc);

#ifdef BRIDGE_DEBUG
	// Open the UART using the coding required.
	uart_open(UART0, 1, UART_DIVIDER_19200_BAUD, uart_data_bits_8, uart_parity_none, uart_stop_bits_1);

	/* Print out a welcome message... */
	BRIDGE_DEBUG_PRINTF("\x1B[2J" /* ANSI/VT100 - Clear the Screen */
			"\x1B[H"  /* ANSI/VT100 - Move Cursor to Home */
	);

	BRIDGE_DEBUG_PRINTF("(C) Copyright 2016, Future Technology Devices International Ltd. \r\n");
	BRIDGE_DEBUG_PRINTF("--------------------------------------------------------------------- \r\n");
	BRIDGE_DEBUG_PRINTF("Welcome to the Ethernet I2C Bridge ... \r\n");
	BRIDGE_DEBUG_PRINTF("\r\n");
	BRIDGE_DEBUG_PRINTF("Connect to FT90x via Ethernet and control EEPROM memory via I2C.\r\n");
	BRIDGE_DEBUG_PRINTF("--------------------------------------------------------------------- \r\n");
#endif // BRIDGE_DEBUG

	sys_enable(sys_device_timer_wdt);

	/* Timer A = 1ms */
	timer_prescaler(1000);
	timer_init(timer_select_a, 100, timer_direction_down, timer_prescaler_select_on, timer_mode_continuous);
	timer_enable_interrupt(timer_select_a);
	timer_start(timer_select_a);

	interrupt_attach(interrupt_timers, (int8_t)interrupt_timers, ISR_timer);

	// Start watchdog timer to restart the device after approx 1.34 seconds.
	wdt_init(wdt_counter_1G_clocks);

	/* Set up I2C */
	i2cm_init(I2CM_NORMAL_SPEED, 10000);

	sys_enable(sys_device_ethernet);

	/* Enable the I2C Slave device... */
	sys_enable(sys_device_i2c_master);

    /* Setup I2C channel 0 pins */
    /* Use sys_i2c_swop(0) to activate. */
    gpio_function(44, pad_i2c0_scl); /* I2C0_SCL */
    gpio_function(45, pad_i2c0_sda); /* I2C0_SDA */

    /* Setup I2C channel 1 pins for EEPROM */
    /* Use sys_i2c_swop(1) to activate. */
    gpio_function(46, pad_i2c1_scl); /* I2C1_SCL */
    gpio_function(47, pad_i2c1_sda); /* I2C1_SDA */


	interrupt_enable_globally();

	i2c_bridge();

	interrupt_disable_globally();

	// Wait forever...
	for (;;);

	return 0;
}
